#ifndef __Vector_h__
#define __Vector_h__

#include <cstdio>

#include "Error.h"
#include <vector>

template<class T>
class Vector
{
    typedef std::vector<T> Super;
public:
    typedef typename Super::iterator iterator;
    typedef typename Super::const_iterator const_iterator;

    virtual void append( T value )
    {
        _container.push_back( value );
    }

    virtual void insertHead( T value )
    {
        _container.insert( _container.begin(), value );
    }

    virtual ~Vector()
    {
    }

    T &operator[]( int index )
    {
        return _container[index];
    }

    bool empty() const
    {
        return size() == 0;
    }

    unsigned size() const
    {
        return _container.size();
    }

    bool exists( const T &value )
    {
        for ( unsigned i = 0; i < size(); ++i )
        {
            if ( _container[i] == value )
            {
                return true;
            }
        }

        return false;
    }

    bool exists( T &value )
    {
        for ( unsigned i = 0; i < size(); ++i )
        {
            if ( _container[i] == value )
            {
                return true;
            }
        }

        return false;
    }

    void erase( const T &value )
    {
        for ( iterator it = _container.begin(); it != _container.end(); ++it )
        {
            if ( (*it) == value )
            {
                _container.erase( it );
                return;
            }
        }

        throw Error( Error::VALUE_DOESNT_EXIST_IN_VECTOR );
    }

    void eraseByValue( T value )
    {
        for ( iterator it = _container.begin(); it != _container.end(); ++it )
        {
            if ( (*it) == value )
            {
                _container.erase( it );
                return;
            }
        }

        throw Error( Error::VALUE_DOESNT_EXIST_IN_VECTOR );
    }

    void eraseAt( unsigned index )
    {
        if ( index >= size() )
        {
            throw Error( Error::VECTOR_OUT_OF_BOUND );
        }

        iterator it = _container.begin();

        while ( index > 0 )
        {
            ++it;
            --index;
        }

        _container.erase( it );
    }

    void clear()
    {
        _container.clear();
    }

    Vector<T> operator+( Vector<T> &other )
    {
        Vector<T> output;

        for ( unsigned i = 0; i < this->size(); ++i )
        {
            output.append( ( *this )[i] );
        }

        for ( unsigned i = 0; i < other.size(); ++i )
        {
            output.append( other[i] );
        }

        return output;
    }

    Vector<T> &operator+=( Vector<T> &other )
    {
        (*this) = (*this) + other;
        return *this;
    }

    void uniqueElements()
    {
        iterator it;

        for ( it = _container.begin(); it != _container.end(); ++it )
        {
            iterator runner = it;
            ++runner;

            while ( runner != _container.end() )
            {
                if ( (*runner) == (*it) )
                {
                    runner = _container.erase( runner );
                }
                else
                {
                    ++runner;
                }
            }
        }
    }

    T popFirst()
    {
        if ( size() == 0 )
        {
            throw Error( Error::POPPING_FROM_EMPTY_VECTOR );
        }

        T value = _container[0];
        eraseAt( 0 );
        return value;
    }

    T last()
    {
        if ( size() == 0 )
        {
            throw Error( Error::VECTOR_OUT_OF_BOUND );
        }

        return _container[size() - 1];
    }

    T pop()
    {
        if ( size() == 0 )
        {
            throw Error( Error::POPPING_FROM_EMPTY_VECTOR );
        }

        T value = last();
        eraseAt( size() - 1 );
        return value;
    }

protected:
    Super _container;
};

#endif // __Vector_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
